@@ -1,26 +1,28 @@
-/*      $Id: menu.c 20529 2006-03-24 14:46:19Z olivier $
- 
+/*      $Id: menu.c 29275 2009-01-18 21:45:33Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
+
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -30,37 +32,47 @@
 #include <gtk/gtk.h>
 #include <unistd.h>
 #include <string.h>
-#include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4util/libxfce4util.h>
+
+#include "event_filter.h"
 #include "menu.h"
+#include "misc.h"
+#include "stock_icons.h"
 
 static GtkWidget *menu_open = NULL;
 static MenuItem menuitems[] = {
-    {MENU_OP_MAXIMIZE, "gtk-zoom-100", N_("Ma_ximize")},
-    {MENU_OP_UNMAXIMIZE, "gtk-zoom-out", N_("Un_maximize")},
-    {MENU_OP_MINIMIZE, "gtk-undo", N_("_Hide")},
-    {MENU_OP_MINIMIZE_ALL, "gtk-clear", N_("Hide _all others")},
-    {MENU_OP_UNMINIMIZE, "gtk-add", N_("S_how")},
-    {MENU_OP_SHADE, "gtk-goto-top", N_("_Shade")},
-    {MENU_OP_UNSHADE, "gtk-goto-bottom", N_("Un_shade")},
-    {MENU_OP_STICK, "gtk-add", N_("S_tick")},
-    {MENU_OP_UNSTICK, "gtk-remove", N_("Uns_tick")},
-    {MENU_OP_CONTEXT_HELP, "gtk-help", N_("Context _help")},
-    {MENU_OP_ABOVE, NULL, N_("Always on top")},
-    {MENU_OP_NORMAL, "gtk-apply", N_("Always on top")},
-    {MENU_OP_WORKSPACES, NULL, N_("Send to...")},
-    {0, NULL, NULL},
-    {MENU_OP_DELETE, "gtk-close", N_("_Close")},
+    {MENU_OP_MAXIMIZE,     WM_STOCK_MAXIMIZE,   N_("Ma_ximize")},
+    {MENU_OP_UNMAXIMIZE,   WM_STOCK_UNMAXIMIZE, N_("Unma_ximize")},
+    {MENU_OP_MINIMIZE,     WM_STOCK_MINIMIZE,   N_("Mi_nimize")},
+    {MENU_OP_MINIMIZE_ALL, NULL,                N_("Minimize _All Other Windows")},
+    {MENU_OP_UNMINIMIZE,   NULL,                N_("S_how")},
+    {MENU_OP_MOVE,         NULL,                N_("_Move")},
+    {MENU_OP_RESIZE,       NULL,                N_("_Resize")},
+    {0, NULL, NULL}, /* -------------------------------------------------------- */
+    {MENU_OP_ABOVE,        NULL,                N_("Always on Top")},
+    {MENU_OP_NORMAL,       NULL,                N_("Same as Other Windows")},
+    {MENU_OP_BELOW,        NULL,                N_("Always Below Other Windows")},
+    {MENU_OP_SHADE,        WM_STOCK_ROLLUP,     N_("Roll Window Up")},
+    {MENU_OP_UNSHADE,      WM_STOCK_ROLLDOWN,   N_("Roll Window Down")},
+    {MENU_OP_FULLSCREEN,   "gtk-fullscreen",    N_("_Fullscreen")},
+    {MENU_OP_UNFULLSCREEN, "gtk-fullscreen",    N_("Leave _Fullscreen")},
+    {MENU_OP_CONTEXT_HELP, "gtk-help",          N_("Context _Help")},
+    {0, NULL, NULL}, /* -------------------------------------------------------- */
+    {MENU_OP_STICK,        NULL,                N_("Always on Visible Workspace")},
+    {MENU_OP_UNSTICK,      NULL,                N_("Only on This Workspace")},
+    {MENU_OP_WORKSPACES,   NULL,                N_("Move to Another Workspace")},
+    {0, NULL, NULL}, /* -------------------------------------------------------- */
+    {MENU_OP_DELETE,       "gtk-close",         N_("_Close")},
 #if 0
-    {0,, NULL NULL},
-    {MENU_OP_DESTROY, "gtk-delete", N_("Destroy")},
-    {0, NULL, NULL},
+    {0, NULL, NULL}, /* -------------------------------------------------------- */
+    {MENU_OP_DESTROY,      "gtk-delete",        N_("Destroy")},
+    {0, NULL, NULL}, /* -------------------------------------------------------- */
 #endif
-    {MENU_OP_QUIT, "gtk-quit", N_("_Quit")},
-    {MENU_OP_RESTART, "gtk-refresh", N_("Restart")},
+    {MENU_OP_QUIT, "gtk-quit",                  N_("_Quit")},
+    {MENU_OP_RESTART, "gtk-refresh",            N_("Restart")},
 };
 
-static XfceFilterStatus
+static eventFilterStatus
 menu_filter (XEvent * xevent, gpointer data)
 {
     switch (xevent->type)
@@ -72,12 +84,12 @@ menu_filter (XEvent * xevent, gpointer data)
         case MotionNotify:
         case EnterNotify:
         case LeaveNotify:
-            return XEV_FILTER_STOP;
+            return EVENT_FILTER_STOP;
             break;
         default:
             break;
     }
-    return XEV_FILTER_CONTINUE;
+    return EVENT_FILTER_CONTINUE;
 }
 
 
@@ -87,6 +99,8 @@ popup_position_func (GtkMenu * menu, gint * x, gint * y, gboolean * push_in,
 {
     GtkRequisition req;
     GdkPoint *pos;
+    gint monitor;
+    GdkScreen *screen;
 
     pos = user_data;
 
@@ -110,6 +124,11 @@ popup_position_func (GtkMenu * menu, gint * x, gint * y, gboolean * push_in,
     {
         *y = (gdk_screen_height () - req.height) / 2;
     }
+
+    screen = gtk_widget_get_screen (GTK_WIDGET(menu));
+    monitor = gdk_screen_get_monitor_at_point (screen, *x, *y);
+    gtk_menu_set_monitor (GTK_MENU (menu), monitor);
+
     g_free (user_data);
 }
 
@@ -126,10 +145,10 @@ activate_cb (GtkWidget * menuitem, gpointer data)
     menudata = data;
 
     TRACE ("deactivating menu_filter");
-    xfce_pop_event_filter (menudata->menu->filter_setup);
-    (*menudata->menu->func) (menudata->menu, 
-                             menudata->op, 
-                             menudata->menu->xid, 
+    eventFilterPop (menudata->menu->filter_setup);
+    (*menudata->menu->func) (menudata->menu,
+                             menudata->op,
+                             menudata->menu->xid,
                              menudata->menu->data,
                              menudata->data);
     return (FALSE);
@@ -144,7 +163,7 @@ menu_closed (GtkMenu * widget, gpointer data)
     menu = data;
     menu_open = NULL;
     TRACE ("deactivating menu_filter");
-    xfce_pop_event_filter (menu->filter_setup);
+    eventFilterPop (menu->filter_setup);
     (*menu->func) (menu, 0, menu->xid, menu->data, NULL);
     return (FALSE);
 }
@@ -152,11 +171,11 @@ menu_closed (GtkMenu * widget, gpointer data)
 static GtkWidget *
 menu_workspace (Menu * menu, MenuOp insensitive, gint ws, gint nws, gchar **wsn, gint wsn_items)
 {
-    gint i;
     GtkWidget *menu_widget;
     GtkWidget *menuitem;
     MenuData *menudata;
     gchar *name;
+    gint i;
 
     menu_widget = gtk_menu_new ();
     gtk_menu_set_screen (GTK_MENU (menu->menu), menu->screen);
@@ -165,20 +184,16 @@ menu_workspace (Menu * menu, MenuOp insensitive, gint ws, gint nws, gchar **wsn,
     {
         if ((i < wsn_items) && wsn[i])
         {
-            name = g_strdup_printf (_("Workspace %i (%s)"), i+ 1, wsn[i]);
+            name = g_strdup_printf ("%i (%s)", i+ 1, wsn[i]);
         }
         else
         {
-            name = g_strdup_printf (_("Workspace %i"), i + 1);
-        }
-        menuitem = gtk_check_menu_item_new_with_label (name);
-        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), (i == ws));
-        gtk_widget_show (menuitem);
-        if (insensitive & MENU_OP_WORKSPACES)
-        {
-            gtk_widget_set_sensitive (menuitem, FALSE);
+            name = g_strdup_printf ("%i", i + 1);
         }
+        menuitem = gtk_menu_item_new_with_label (name);
         g_free (name);
+        gtk_widget_set_sensitive (menuitem, !(insensitive & MENU_OP_WORKSPACES) && (i != ws));
+        gtk_widget_show (menuitem);
 
         menudata = g_new (MenuData, 1);
         menudata->menu = menu;
@@ -193,11 +208,16 @@ menu_workspace (Menu * menu, MenuOp insensitive, gint ws, gint nws, gchar **wsn,
 }
 
 Menu *
-menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuFunc func, 
-    gint ws, gint nws, gchar **wsn, gint wsn_items, XfceFilterSetup *filter_setup, gpointer data)
+menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuFunc func,
+    gint ws, gint nws, gchar **wsn, gint wsn_items, eventFilterSetup *filter_setup, gpointer data)
 {
-    int i;
+    GtkWidget *menuitem;
+    GtkWidget *image;
+    GtkWidget *ws_menu;
+    MenuData *menudata;
     Menu *menu;
+    const gchar *label;
+    int i;
 
     TRACE ("entering menu_new");
     menu = g_new (Menu, 1);
@@ -216,12 +236,6 @@ menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuF
     {
         if ((ops & menuitems[i].op) || (menuitems[i].op == MENU_OP_SEPARATOR))
         {
-            GtkWidget *menuitem;
-            GtkWidget *image;
-            GtkWidget *ws_menu;
-            MenuData *menudata;
-            const gchar *label;
-
             label = _(menuitems[i].label);
             ws_menu = NULL;
             switch (menuitems[i].op)
@@ -331,20 +345,22 @@ menu_check_and_close (void)
 static gboolean
 grab_available (GdkWindow *win, guint32 timestamp)
 {
-    GdkEventMask mask =
-        GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-        GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
-        GDK_POINTER_MOTION_MASK;
+    GdkEventMask mask;
     GdkGrabStatus g1;
     GdkGrabStatus g2;
-    gboolean grab_failed = FALSE;
-    gint i = 0;
+    gboolean grab_failed;
+    gint i;
 
     TRACE ("entering grab_available");
 
+    mask = GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+           GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
+           GDK_POINTER_MOTION_MASK;
     g1 = gdk_pointer_grab (win, TRUE, mask, NULL, NULL, timestamp);
     g2 = gdk_keyboard_grab (win, TRUE, timestamp);
+    grab_failed = FALSE;
 
+    i = 0;
     while ((i++ < 100) && (grab_failed = ((g1 != GDK_GRAB_SUCCESS)
                 || (g2 != GDK_GRAB_SUCCESS))))
     {
@@ -397,7 +413,7 @@ menu_popup (Menu * menu, int root_x, int root_y, int button,
         }
         TRACE ("opening new menu");
         menu_open = menu->menu;
-        xfce_push_event_filter (menu->filter_setup, menu_filter, NULL);
+        eventFilterPush (menu->filter_setup, menu_filter, NULL);
         gtk_menu_popup (GTK_MENU (menu->menu), NULL, NULL,
             popup_position_func, pt, 0, timestamp);
 
@@ -407,7 +423,7 @@ menu_popup (Menu * menu, int root_x, int root_y, int button,
             g_message (_("%s: GtkMenu failed to grab the pointer\n"), g_get_prgname ());
             gtk_menu_popdown (GTK_MENU (menu->menu));
             menu_open = NULL;
-            xfce_pop_event_filter (menu->filter_setup);
+            eventFilterPop (menu->filter_setup);
             return FALSE;
         }
     }
@@ -418,10 +434,13 @@ void
 menu_free (Menu * menu)
 {
     TRACE ("entering menu_free");
+
     g_return_if_fail (menu != NULL);
     g_return_if_fail (menu->menu != NULL);
     g_return_if_fail (GTK_IS_MENU (menu->menu));
+
     TRACE ("freeing menu");
+
     gtk_widget_destroy (menu->menu);
     g_free (menu);
 }
