@@ -1,26 +1,28 @@
-/*      $Id: session.c,v 1.24 2004/11/23 23:57:33 olivier Exp $
- 
+/*      $Id: session.c 29275 2009-01-18 21:45:33Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
+
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
  */
 
 /* Initially inspired by xfwm, fvwm2, enlightment and twm implementations */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xatom.h>
@@ -36,6 +38,10 @@
 #include <sys/types.h>
 #include <signal.h>
 
+#include <gtk/gtk.h>
+#include <glib.h>
+#include <libxfcegui4/libxfcegui4.h>
+
 #include "display.h"
 #include "screen.h"
 #include "hints.h"
@@ -96,7 +102,7 @@ my_free_string_list (gchar ** list, gint n)
 
 /*
    2-pass function to compute new string length,
-   allocate memory and finally copy string 
+   allocate memory and finally copy string
    - Returned value must be freed -
  */
 static gchar *
@@ -104,11 +110,12 @@ escape_quote (gchar * s)
 {
     gchar *ns;
     gchar *idx1, *idx2;
-    gint nbquotes = 0;
-    gint lg = 0;
+    gint nbquotes, lg;
 
     g_return_val_if_fail (s != NULL, NULL);
 
+    nbquotes = 0;
+    lg = 0;
     lg = strlen (s);
     /* First, count quotes in string */
     idx1 = s;
@@ -150,7 +157,7 @@ escape_quote (gchar * s)
 
 /*
    single-pass function to replace backslash+quotes
-   by quotes. 
+   by quotes.
    - Returned value must be freed -
  */
 static gchar *
@@ -197,12 +204,13 @@ static gchar *
 getsubstring (gchar * s, gint * length)
 {
     gchar pbrk;
-    gchar *ns;
-    gchar *end, *idx1, *idx2, *skip;
+    gchar *ns, *end, *idx1, *idx2, *skip;
     gint lg;
-    gboolean finished = FALSE, backslash = FALSE;
+    gboolean finished, backslash;
 
     lg = *length = 0;
+    finished = FALSE;
+    backslash = FALSE;
     g_return_val_if_fail (s != NULL, NULL);
 
     end = skip = s;
@@ -258,18 +266,27 @@ getsubstring (gchar * s, gint * length)
 static void
 sessionSaveScreen (ScreenInfo *screen_info, FILE *f)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     Client *c;
+    char *client_id, *window_role;
+    char **wm_command;
+    int wm_command_count;
     gint client_idx;
-    char *client_id = NULL;
-    char *window_role = NULL;
-    int wm_command_count = 0;
-    char **wm_command = NULL;
-    
+
     display_info = screen_info->display_info;
+    wm_command_count = 0;
+    wm_command = NULL;
+    window_role = NULL;
+    client_id = NULL;
+
     for (c = screen_info->clients, client_idx = 0; client_idx < screen_info->client_count;
         c = c->next, client_idx++)
     {
+        if (c->type & (WINDOW_TYPE_DONT_PLACE))
+        {
+            continue;
+        }
+
         if (c->client_leader != None)
         {
             getWindowRole (display_info, c->window, &window_role);
@@ -490,6 +507,7 @@ void
 sessionFreeWindowStates (void)
 {
     int i;
+
     for (i = 0; i < num_match; i++)
     {
         if (matches[i].client_id)
@@ -541,21 +559,24 @@ sessionFreeWindowStates (void)
 static gboolean
 matchWin (Client * c, Match * m)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    char *client_id = NULL;
-    char *window_role = NULL;
-    int wm_command_count = 0;
-    char **wm_command = NULL;
-    gboolean found;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    char *client_id;
+    char *window_role;
+    char **wm_command;
+    int wm_command_count;
     int i;
+    gboolean found;
 
     g_return_val_if_fail (c != NULL, FALSE);
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
-
+    wm_command = NULL;
+    window_role = NULL;
+    client_id = NULL;
     found = FALSE;
+
     getClientID (display_info, c->window, &client_id);
     if (xstreq (client_id, m->client_id))
     {
@@ -575,7 +596,7 @@ matchWin (Client * c, Match * m)
         }
         else
         {
-            /* 
+            /*
              * Compare res_class, res_name and WM_NAME, unless the
              * WM_NAME has changed
              */
@@ -607,8 +628,8 @@ matchWin (Client * c, Match * m)
                         {
                             found = TRUE;
                         }
-                    }           /* if (wm_command_count ==... */
-                    /* 
+                    }
+                    /*
                      * We have to deal with a now-SM-aware client, it means that it won't probably
                      * restore its state in a proper manner.
                      * Thus, we also mark all other instances of this application as used, to avoid
@@ -683,3 +704,93 @@ sessionMatchWinToSM (Client * c)
     }
     return FALSE;
 }
+
+static char *
+sessionBuildFilename(SessionClient *client_session)
+{
+    gchar *filename, *path, *file;
+    GError *error;
+
+    path = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "sessions", FALSE);
+
+    error = NULL;
+    if (!xfce_mkdirhier(path, 0700, &error))
+    {
+        g_warning("Unable to create session dir %s: %s", path, error->message);
+        g_error_free (error);
+        g_free (path);
+        return NULL;
+    }
+
+    file = g_strdup_printf("xfwm4-%s", client_session->given_client_id);
+    filename = g_build_filename (path, file, NULL);
+    g_free (file);
+    g_free (path);
+
+    return filename;
+}
+
+static void
+sessionLoad (DisplayInfo *display_info)
+{
+    SessionClient *session;
+    gchar *filename;
+
+    session = display_info->session;
+    filename = sessionBuildFilename(session);
+    if (filename)
+    {
+        sessionLoadWindowStates (filename);
+        g_free (filename);
+    }
+}
+
+static void
+sessionSavePhase2 (gpointer data)
+{
+    DisplayInfo *display_info;
+    SessionClient *session;
+    gchar *filename;
+
+    display_info = (DisplayInfo *) data;
+    session = display_info->session;
+    filename = sessionBuildFilename(session);
+    if (filename)
+    {
+        sessionSaveWindowStates (display_info, filename);
+        g_free (filename);
+    }
+}
+
+static void
+sessionDie (gpointer data)
+{
+    DisplayInfo *display_info;
+
+    display_info = (DisplayInfo *) data;
+    client_session_set_restart_style(display_info->session, SESSION_RESTART_IF_RUNNING);
+    display_info->quit = TRUE;
+    gtk_main_quit ();
+}
+
+int
+sessionStart (int argc, char **argv, DisplayInfo *display_info)
+{
+    SessionClient *session;
+
+    display_info->session = client_session_new (argc, argv, (gpointer) display_info,
+                                                SESSION_RESTART_IMMEDIATELY, 20);
+    session = display_info->session;
+    session->data = (gpointer) display_info;
+    session->save_phase_2 = sessionSavePhase2;
+    session->die = sessionDie;
+
+    if (session_init (session))
+    {
+        sessionLoad (display_info);
+        return 1;
+    }
+
+    return 0;
+}
+
