@@ -1,4 +1,4 @@
-/*      $Id: stacking.c 25627 2007-04-22 21:04:34Z olivier $
+/*      $Id: stacking.c 29275 2009-01-18 21:45:33Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,12 +12,18 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
@@ -34,6 +40,8 @@
 #include "frame.h"
 #include "focus.h"
 
+static guint raise_timeout = 0;
+
 void
 clientApplyStackList (ScreenInfo *screen_info)
 {
@@ -242,7 +250,7 @@ clientGetBottomMost (ScreenInfo *screen_info, int layer, Client * exclude)
  * conditions caused by querying the X server
  */
 Client *
-clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
+clientAtPosition (ScreenInfo *screen_info, int x, int y, GList * exclude_list)
 {
     GList *index;
     Client *c, *c2;
@@ -257,7 +265,7 @@ clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
             && (frameY (c2) <= y) && (frameY (c2) + frameHeight (c2) >= y))
         {
             if (clientSelectMask (c2, INCLUDE_SKIP_PAGER | INCLUDE_SKIP_TASKBAR, WINDOW_REGULAR_FOCUSABLE)
-                && (c2 != exclude))
+                && !g_list_find (exclude_list, (gconstpointer) c2))
             {
                 c = c2;
                 break;
@@ -328,7 +336,7 @@ clientRaise (Client * c, Window wsibling)
         /* Search for the window that will be just on top of the raised window  */
         if (wsibling)
         {
-            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, SEARCH_FRAME | SEARCH_WINDOW);
             if (c2)
             {
                 sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
@@ -475,7 +483,7 @@ clientLower (Client * c, Window wsibling)
         }
         else if (wsibling)
         {
-            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, SEARCH_FRAME | SEARCH_WINDOW);
             if (c2)
             {
                 sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
@@ -541,11 +549,11 @@ clientAdjustFullscreenLayer (Client *c, gboolean set)
         if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
         {
-            clientSetLayer (c, WIN_LAYER_ABOVE_DOCK);
+            clientSetLayer (c, WIN_LAYER_FULLSCREEN);
             return TRUE;
         }
     }
-    else if (c->win_layer == WIN_LAYER_ABOVE_DOCK)
+    else if (c->win_layer == WIN_LAYER_FULLSCREEN)
     {
         if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
@@ -682,3 +690,44 @@ clientClearLastRaise (ScreenInfo *screen_info)
     g_return_if_fail (screen_info);
     screen_info->last_raise = NULL;
 }
+
+static gboolean
+delayed_raise_cb (gpointer data)
+{
+    Client *c;
+
+    TRACE ("entering delayed_raise_cb");
+
+    clientClearDelayedRaise ();
+    c = clientGetFocus ();
+
+    if (c)
+    {
+        clientRaise (c, None);
+    }
+    return (TRUE);
+}
+
+void
+clientClearDelayedRaise (void)
+{
+    if (raise_timeout)
+    {
+        g_source_remove (raise_timeout);
+        raise_timeout = 0;
+    }
+}
+
+void
+clientResetDelayedRaise (ScreenInfo *screen_info)
+{
+    if (raise_timeout)
+    {
+        g_source_remove (raise_timeout);
+    }
+    raise_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
+                                        screen_info->params->raise_delay,
+                                        (GSourceFunc) delayed_raise_cb,
+                                        NULL, NULL);
+}
+
